﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;
using System.Diagnostics;

namespace bmake
{
    class Program
    {
        const string VersionString = "1.0.1";
        static int Main(string[] args)
        {
            //設定ファイルロード
            Utilities.TA_INIDocument ini = new Utilities.TA_INIDocument("bmake.ini");
            try
            {
                ini["Path", "BMSLN_CONV"] = Environment.ExpandEnvironmentVariables(ini["Path", "BMSLN_CONV"]);
                ini["Path", "MODEL_CONV"] = Environment.ExpandEnvironmentVariables(ini["Path", "MODEL_CONV"]);
                ini["Path", "TEXTURE_CONV"] = Environment.ExpandEnvironmentVariables(ini["Path", "TEXTURE_CONV"]);
            }
            catch (Exception e)
            {
                Console.WriteLine("!- bmake.iniの読み込み時にエラーが発生しました。\n!- {0}", e.ToString());
                return 2;
            }
            string CfgTexturePath = Path.Combine(ini["Path", "DATABASE"], "texture");
            string CfgModelPath = Path.Combine(ini["Path", "DATABASE"], "model");
            string CfgModelTexturePath = Path.Combine(ini["Path", "DATABASE"], "model");
            //bmslnファイル読み込み……の前にコマンドライン確認
            if (args.Length != 1)
            {
                Console.WriteLine(@"usage: bmake hoge.bmsln
bmake v" + VersionString + @" (C)2012 D.N.A.Softwares
BISHAMONのエフェクトデータを解析し関連するテクスチャとモデルのみを変換するツールです。
各種パス・コマンドラインは起動フォルダのbmake.iniで設定します。");
                return 1;
            }
            //bmsln読み込み

            List<string> lTextureNames = new List<string>();
            List<string> lModelNames = new List<string>();
            List<string> lModelTextureNames = new List<string>();

            FileStream fsBmsln = new FileStream(args[0], FileMode.Open, FileAccess.Read);
            XmlTextReader xrBmsln = new XmlTextReader(fsBmsln);
            Console.WriteLine("-- 画像ファイル・モデルファイル列挙.");
            while (xrBmsln.Read())
            {
                xrBmsln.MoveToContent();

                if (xrBmsln.NodeType == XmlNodeType.Element)
                {
                    if (xrBmsln.LocalName.Contains("PictureName"))
                    {
                        string n = xrBmsln.ReadString();
                        string PicNameCandidate = n;
                        if (File.Exists(Path.Combine(CfgTexturePath, n + ".png")))
                            PicNameCandidate = n + ".png";
                        else if (File.Exists(Path.Combine(CfgTexturePath, n + ".tga")))
                            PicNameCandidate = n + ".tga";
                        else if (File.Exists(Path.Combine(CfgTexturePath, n + ".dds")))
                            PicNameCandidate = n + ".dds";
                            
                        if (n.Length > 0 && !lTextureNames.Contains(PicNameCandidate))
                        {
                            lTextureNames.Add(PicNameCandidate);
                            Console.WriteLine("---- pic : {0}", PicNameCandidate);
                        }
                    }

                    if (xrBmsln.LocalName.Contains("ModelName"))
                    {
                        string n = xrBmsln.ReadString();
                        if (n.Length > 0 && !lModelNames.Contains(n))  
                        {
                            lModelNames.Add(n);
                            Console.WriteLine("---- mdl : {0}", n);
                        }
                    }
                }
            }
            xrBmsln.Close();
            fsBmsln.Close();
            Console.WriteLine("-- bmslnの解析完了.");

            //モデルデータからテクスチャ名抽出
            foreach (string modelname in lModelNames)
            {
                string modelpath = Path.Combine(CfgModelPath, modelname + ".dae");
                try
                {
                    FileStream fsModel = new FileStream(modelpath, FileMode.Open, FileAccess.Read);
                    XmlTextReader xrModel = new XmlTextReader(fsModel);
                    Console.WriteLine("-- {0} のテクスチャを列挙.",modelpath);
                    while (xrModel.Read())
                    {
                        xrModel.MoveToContent();
                        if (xrModel.NodeType == XmlNodeType.Element)
                        {
                            if (xrModel.LocalName.Contains("init_from"))
                            {
                                //ファイル名かもしれないがファイル名でない可能性もあるので、ファイル名（らしきもの）を抽出し
                                //そのファイルの所在を確認する。
                                string n = xrModel.ReadString();
                                string candidateFileName = n;
                                Uri u;
                                if (Uri.TryCreate(n,UriKind.Absolute,out u))
                                {
                                    candidateFileName = u.Segments[u.Segments.Length - 1];
                                }
                                else
                                {
                                    if (Uri.TryCreate("file:///" + n, UriKind.RelativeOrAbsolute, out u))
                                    {
                                        candidateFileName = u.Segments[u.Segments.Length - 1];
                                    }
                                }
                                if (File.Exists(Path.Combine(CfgModelTexturePath, candidateFileName)))
                                {
                                    if (!lModelTextureNames.Contains(candidateFileName))
                                    {
                                        lModelTextureNames.Add(candidateFileName);
                                        Console.WriteLine("---- {0}",candidateFileName);
                                    }
                                }
                            }
                        }
                    }
                    xrModel.Close();
                    fsModel.Close();
                    Console.WriteLine("-- 終了.");
                }
                catch (FileNotFoundException e)
                {
                    Console.WriteLine("!- モデルファイル {0} の読み込みに失敗",e.FileName);
                    continue;
                }
            }
            Console.WriteLine("-- bmslnを変換...");
            Process p;
            ProcessStartInfo ps = new ProcessStartInfo(ini["Path", "BMSLN_CONV"], string.Format(ini["Command", "BMSLN_CONV_ARGS"], args[0]));
            ps.UseShellExecute = false;
            p = Process.Start(ps);
            p.WaitForExit();
            string bmb = Path.GetFileNameWithoutExtension(args[0]) + ".bmb";
            FileMoveForceOverwrite(
                Path.Combine(Path.GetDirectoryName(args[0]),bmb),
                Path.Combine(ini["Path","OUT_BMB"],bmb)
                );
            
            Console.WriteLine("-- テクスチャを変換...");
            ps.FileName=ini["Path", "TEXTURE_CONV"];
            foreach (string tfile in lTextureNames)
            {
                ps.Arguments = string.Format(ini["Command", "TEXTURE_CONV_ARGS"], Path.Combine(CfgTexturePath, tfile));
                p = Process.Start(ps);
                p.WaitForExit();
                string dds = Path.GetFileNameWithoutExtension(tfile) + ".dds";
                if (File.Exists(Path.Combine(CfgTexturePath, dds)))
                {
                    FileMoveForceOverwrite(
                        Path.Combine(CfgTexturePath, dds),
                        Path.Combine(ini["Path", "OUT_TEXTURE"], dds)
                    );
                }
            }
            Console.WriteLine("-- モデル用テクスチャを変換...");
//            ps.FileName = ini["Path", "TEXTURE_CONV"];
            foreach (string tfile in lModelTextureNames)
            {
                ps.Arguments=string.Format(ini["Command", "MODEL_TEXTURE_CONV_ARGS"], Path.Combine(CfgModelTexturePath, tfile));
                p = Process.Start(ps);
                p.WaitForExit();
                string dds = Path.GetFileNameWithoutExtension(tfile) + ".dds";
                if (File.Exists(Path.Combine(CfgModelTexturePath, dds)))
                {
                    FileMoveForceOverwrite(
                        Path.Combine(CfgModelTexturePath, dds),
                        Path.Combine(ini["Path", "OUT_MODEL_TEXTURE"], dds)
                    );
                }
            }
            Console.WriteLine("-- モデルを変換...");
            ps.FileName=ini["Path", "MODEL_CONV"];
            foreach (string tfile in lModelNames)
            {
                ps.Arguments = string.Format(ini["Command", "MODEL_CONV_ARGS"], Path.Combine(CfgModelPath, tfile+".dae"));
                p = Process.Start(ps);
                p.WaitForExit();
                string m3r = Path.GetFileNameWithoutExtension(tfile) + ".m3r";
                if (File.Exists(Path.Combine(CfgModelPath, m3r)))
                {
                    FileMoveForceOverwrite(
                        Path.Combine(CfgModelPath, m3r),
                        Path.Combine(ini["Path", "OUT_MODEL"], m3r)
                    );
                }
            }
            Console.WriteLine("-- 正常終了.");

#if DEBUG
            Console.ReadLine();
#endif
            return 0;
        }
        static bool FileMoveForceOverwrite(string src, string dest)
        {
            bool flag = false;
            if (File.Exists(dest))
            {
                File.Delete(dest);
                flag = true;
            }
            File.Move(src, dest);
            return flag ;
        }
    }
}
